/*	-*- C++ -*-
 * Copyright (C) 2001,  Christof Meerwald <cmeerw@web.de>
 * http://xpcom2corba.sunsite.dk/
 */

/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License
 * at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and
 * limitations under the License.
 *
 * The Original Code is the XPCOM-CORBA Bridge package.
 *
 * The Initial Developer of the Original Code is Christof Meerwald.
 * Portions created by Christof Meerwald are Copyright (C) 2001
 * Christof Meerwald.  All Rights Reserved.
 *
 * Contributor(s): Christof Meerwald <cmeerw@web.de> (original author)
 */

#include "corba4xpcom_impl.h"
#include "corba_poa.h"
#include "xpcom_object.h"

#include <nsMemory.h>
#include <nspr.h>

#include <iostream>
#include <string>

#include <nsIDirectoryService.h>
#ifndef MOZILLA_INTERNAL_API
#include <nsStringAPI.h>
#include <nsIProperties.h>
#include <nsServiceManagerUtils.h>
#else
#include <nsString.h>
#endif

#ifdef MOZILLA_INTERNAL_API

#endif
char *CORBA_ORBProxy_Impl::ORB_argv_[] = {"CORBA_ORBProxy_Impl",
#ifdef WIN32
			  "-ORBendPoint", "giop:tcp:127.0.0.1:0",
#else
                          "-ORBendPoint", "giop:unix:",
                          "-ORBunixTransportPermission", "0700",
#endif
#ifdef HIGH_ORBTRACELEVEL
                          "-ORBtraceLevel", "2000",
#endif
					  NULL
                                         };
int CORBA_ORBProxy_Impl::ORB_argc_ =
  sizeof (CORBA_ORBProxy_Impl::ORB_argv_) / sizeof (char *) - 1;


NS_IMPL_THREADSAFE_ISUPPORTS1(CORBA_ORBProxy_Impl, CORBA_IORBProxy)

CORBA_ORBProxy_Impl::CORBA_ORBProxy_Impl()
  : orb_(CORBA::ORB_init(ORB_argc_, ORB_argv_))
{
  NS_INIT_ISUPPORTS();

  /* member initializers and constructor code */
}

CORBA_ORBProxy_Impl::~CORBA_ORBProxy_Impl()
{
  /* destructor code */
}


/* boolean load_module (in string name); */
NS_IMETHODIMP CORBA_ORBProxy_Impl::Load_module(
                                               const char *package_uuid,
                                               const char *name,
					       PRBool *_retval)
{
  nsresult rv;
  nsCOMPtr<nsIProperties> ds =
    do_GetService("@mozilla.org/file/directory_service;1", &rv);
  NS_ENSURE_SUCCESS(rv, rv);
#ifndef MOZILLA_INTERNAL_API
  nsCString comdstr;
#else
  nsCAutoString comdstr;
#endif

  nsCOMPtr<nsIFile> file;
  rv = ds->Get("ComsD", NS_GET_IID(nsIFile), getter_AddRefs(file));
  NS_ENSURE_SUCCESS(rv, rv);
#ifndef MOZILLA_INTERNAL_API
  nsCString fullname(NS_LITERAL_CSTRING("xpcorba_"));
#else
  nsCAutoString fullname(NS_LITERAL_CSTRING("xpcorba_"));
#endif
  fullname += name;
#ifdef XP_WIN
  fullname += ".dll";
#elif XP_MACOSX
  fullname += ".dylib";
#else
  fullname += ".so";
#endif
  file->AppendNative(fullname);
  file->GetNativePath(comdstr);

  *_retval = PR_LoadLibrary(comdstr.get()) ? PR_TRUE : PR_FALSE;
  if (*_retval == PR_FALSE)
  {
    rv = ds->Get("ProfD", NS_GET_IID(nsIFile), getter_AddRefs(file));
    NS_ENSURE_SUCCESS(rv, rv);
#ifndef MOZILLA_INTERNAL_API
    nsCString fullname(NS_LITERAL_CSTRING("xpcorba_"));
#else
    nsCAutoString fullname(NS_LITERAL_CSTRING("xpcorba_"));
#endif
    fullname += name;
#ifdef XP_WIN
    fullname += ".dll";
#else
    fullname += ".so";
#endif
    file->Append(NS_LITERAL_STRING("extensions"));
#ifndef MOZILLA_INTERNAL_API
    nsCString pustr(package_uuid);
#else
    nsCAutoString pustr(package_uuid);
#endif
    file->AppendNative(pustr);
    file->Append(NS_LITERAL_STRING("components"));
    file->AppendNative(fullname);
    file->GetNativePath(comdstr);
    
    *_retval = PR_LoadLibrary(comdstr.get()) ? PR_TRUE : PR_FALSE;
  }

  if (*_retval == PR_FALSE)
    return NS_ERROR_FAILURE;
  return NS_OK;
}


/* nsISupports resolve_initial_references (in string name); */
NS_IMETHODIMP
CORBA_ORBProxy_Impl::Resolve_initial_references(const char *name,
						nsISupports **_retval)
{
  try
  {
    CORBA::Object_var obj = orb_->resolve_initial_references(name);

    *_retval = new CORBA_IObject__XPCOM_Proxy(obj);
    NS_ADDREF(*_retval);

    return NS_OK;
  }
  catch (const CORBA::UserException &ex)
  {
    // TODO: user exception
  }
  catch (const CORBA::SystemException &ex)
  {
    // TODO: system exception
  }

  return NS_ERROR_UNEXPECTED;
}


/* nsISupports string_to_object (in string ior); */
NS_IMETHODIMP
CORBA_ORBProxy_Impl::String_to_object(const char *ior,
				      nsISupports **_retval)
{
  try
  {
    CORBA::Object_var obj = orb_->string_to_object(ior);

    try
    {
      printf("In String_to_object: narrowing to XPCOM::IObject\n");
      XPCOM::IObject_var xobj = XPCOM::IObject::_narrow(obj);
      printf("Narrow worked, saving into a new XPCOM_IObject__XPCOM_Proxy.\n");
      xobj->add_ref();
      *_retval = new XPCOM_IObject__XPCOM_Proxy(xobj);
      NS_ADDREF(*_retval);
    }
    catch (CORBA::Exception& e)
    {
      printf("Narrow failed.\n");
      *_retval = new CORBA_IObject__XPCOM_Proxy(obj);
      NS_ADDREF(*_retval);
    }

    return NS_OK;
  }
  catch (...)
  {
    // TODO: exception
  }

  return NS_ERROR_UNEXPECTED;
}

CORBA_POA* get_corba_poa();

/* string object_to_string (in nsISupports obj); */
NS_IMETHODIMP
CORBA_ORBProxy_Impl::Object_to_string(nsISupports *obj,
				      char **_retval)
{
  try
  {
    CORBA::Object_var obj_conv;
    CORBA::Object* dummy = (CORBA::Object*)NULL;
    obj_conv = get_corba_poa()->register_object(obj, dummy);

    CORBA::String_var str_conv = orb_->object_to_string(obj_conv);

    *_retval = (char *) nsMemory::Clone(str_conv, sizeof (char) * (strlen(str_conv) + 1));

    return NS_OK;
  }
  catch (const CORBA::UserException &ex)
  {
    // TODO: user exception
  }
  catch (const CORBA::SystemException &ex)
  {
    // TODO: system exception
  }

  return NS_ERROR_UNEXPECTED;
}


/* void run (); */
NS_IMETHODIMP
CORBA_ORBProxy_Impl::Run()
{
  try
  {
    orb_->run();

    return NS_OK;
  }
  catch (const CORBA::UserException &ex)
  {
    // TODO: user exception
  }
  catch (const CORBA::SystemException &ex)
  {
    // TODO: system exception
  }

  return NS_ERROR_UNEXPECTED;
}
